@@ -1,11 +1,10 @@
-use strict;
-use warnings;
+use strictures 2;
 package JSON::Schema::Modern::Document::OpenAPI;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: One OpenAPI v3.1 document
 # KEYWORDS: JSON Schema data validation request response OpenAPI
 
-our $VERSION = '0.049';
+our $VERSION = '0.050';
 
 use 5.020;
 use Moo;
@@ -25,7 +24,6 @@ use Ref::Util 'is_plain_hashref';
 use MooX::HandlesVia;
 use MooX::TypeTiny 0.002002;
 use Types::Standard qw(InstanceOf HashRef Str Enum);
-use Storable 'dclone';
 use namespace::clean;
 
 extends 'JSON::Schema::Modern::Document';
@@ -45,6 +43,7 @@ use constant DEFAULT_SCHEMAS => {
 use constant DEFAULT_BASE_METASCHEMA => 'https://spec.openapis.org/oas/3.1/schema-base/2022-10-07';
 use constant DEFAULT_METASCHEMA => 'https://spec.openapis.org/oas/3.1/schema/2022-10-07';
 
+# warning: this is weak_ref'd, so it may not exist after construction/traverse time
 has '+evaluator' => (
   required => 1,
 );
@@ -245,26 +244,6 @@ sub traverse ($self, $evaluator) {
   return $state;
 }
 
-sub recursive_get ($self, $uri_reference) {
-  my $base = $self->canonical_uri;
-  my $ref = $uri_reference;
-  my ($depth, $schema);
-
-  while ($ref) {
-    die 'maximum evaluation depth exceeded' if $depth++ > $self->evaluator->max_traversal_depth;
-    my $uri = Mojo::URL->new($ref)->to_abs($base);
-
-    my $schema_info = $self->evaluator->_fetch_from_uri($uri);
-    die('unable to find resource ', $uri) if not $schema_info;
-    $schema = $schema_info->{schema};
-    $base = $schema_info->{canonical_uri};
-    $ref = $schema->{'$ref'};
-  }
-
-  $schema = dclone($schema);
-  return wantarray ? ($schema, $base) : $schema;
-}
-
 ######## NO PUBLIC INTERFACES FOLLOW THIS POINT ########
 
 sub _add_vocab_and_default_schemas ($self) {
@@ -341,7 +320,7 @@ JSON::Schema::Modern::Document::OpenAPI - One OpenAPI v3.1 document
 
 =head1 VERSION
 
-version 0.049
+version 0.050
 
 =head1 SYNOPSIS
 
@@ -416,26 +395,6 @@ Returns the json pointer location of the operation containing the provided C<ope
 for passing to C<< $document->get(..) >>), or C<undef> if the location does not exist in the
 document.
 
-=head2 recursive_get
-
-Given a uri or uri-reference, get the definition at that location, following any C<$ref>s along the
-way. Returns the data in scalar context, or a tuple of the data and the canonical URI of the
-referenced location in list context.
-
-If the provided location is relative, the current document is used as the base URI.
-If you have a local json pointer you want to resolve, you can turn it into a uri-reference by
-prepending C<#>.
-
-  # starts with a JSON::Schema::Modern::Document::OpenAPI object
-  $document->recursive_get('#/components/parameters/Content-Encoding');
-
-  # starts with an OpenAPI::Modern object
-  $openapi->openapi_document->recursive_get('#/components/parameters/Content-Encoding');
-
-  # starts with a JSON::Schema::Modern object (TODO)
-  $js->recursive_get('https:///openapi_doc.yaml#/components/schemas/my_object')
-  $js->recursive_get('https://localhost:1234/my_spec#/$defs/my_object')
-
 =head1 SEE ALSO
 
 =over 4
